'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.babylon = undefined;
exports.locateBabelConfig = locateBabelConfig;
exports.getPluginConfig = getPluginConfig;
exports.getCompilerConfiguration = getCompilerConfiguration;
exports.parse = parse;

var _parseConfigurationDirectives = require('./util/parseConfigurationDirectives');

var _parseConfigurationDirectives2 = _interopRequireDefault(_parseConfigurationDirectives);

var _config = require('./config');

var _babylonLightscript = require('@oigroup/babylon-lightscript');

var babylon = _interopRequireWildcard(_babylonLightscript);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var findBabelConfig = require('find-babel-config'); // API for external tools, such as the linter, that share functionality with
// the compiler.


function findConfigEntry(babelConfig, type, pluginName) {
  for (var _arr2 = babelConfig[type] || [], _i3 = 0, _len3 = _arr2.length; _i3 < _len3; _i3++) {
    var _p = _arr2[_i3];
    if (_p === pluginName) {
      return {};
    } else if ((_p == null ? void 0 : _p[0]) === pluginName) {
      return _p[1];
    }
  }
} // Given a file path, locate the applicable .babelrc and extract the options
// pertinent to the LightScript plugin.
function locateBabelConfig(filePath) {
  // Check if filePath is nonsense
  if (!filePath || filePath === "unknown" || filePath === "repl") {
    return null;
  } // Use find-babel-config
  var confData = findBabelConfig.sync(filePath);
  return confData.config;
}function getPluginConfig(babelConfig) {
  var preset = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : "@oigroup/babel-preset-lightscript";
  var plugin = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : "@oigroup/babel-plugin-lightscript";

  if (!babelConfig) return {};
  var entry = findConfigEntry(babelConfig, "presets", preset);
  if (!entry) entry = findConfigEntry(babelConfig, "plugins", plugin);

  return entry || {};
} // Obtain compiler configuration information using the same algorithms used
// by the compiler itself. Loads .babelrc and configuration directives.
function getCompilerConfiguration(filePath, code) {
  var opts = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};

  var pluginOpts = getPluginConfig(locateBabelConfig(filePath), opts.preset, opts.plugin);

  // Merge special opts
  for (var _i4 = 0, _keys2 = Object.keys(opts), _len4 = _keys2.length; _i4 < _len4; _i4++) {
    var _k = _keys2[_i4];var _v = opts[_k];
    if (_k !== "preset" && _k !== "plugin") {
      pluginOpts[_k] = _v;
    }
  }var fileTypeInfo = (0, _config.getFileTypeInfoFromFileName)(filePath);
  if (fileTypeInfo.isLightScript && pluginOpts.isLightScript === undefined) {
    pluginOpts.isLightScript = true;
  }var directiveOpts = (0, _parseConfigurationDirectives2.default)(code);
  Object.assign(pluginOpts, directiveOpts);

  return pluginOpts;
} // Parse code, including configuration directives, generating parser options
// the same way the compiler does. Returns a babylon ast.
function parse(compilerConfig, code, baseParserOpts) {
  var parserOpts = (0, _config.getParserOpts)(compilerConfig, baseParserOpts);
  return parserOpts.parser(code, parserOpts);
} // Let tools have access to babylon if they need it.
exports.babylon = babylon;